require "erb"

$projects = {}

def project name = File.basename(Dir.getwd)
    if $projects.has_key?(name)
        puts "project's name(#{name}) is exists"
        return
    end
    p = Project.new(target: name)
    yield p
    $projects[name] = p
end

class Project
    def self.do_all
        $projects.values.each{|p| p.create_makefile_and_make }
    end

    def self.do name
        $projects[name].create_makefile_and_make if $projects.has_key?(name)
    end

    def self.all
        $projects.keys
    end

    def initialize(options = {})
        @options = {
            makefile: 'Makefile',
            obj_files: 'build/tmp/objects',
            source_files: [],
            include_path: ['include', 'inc', './'],
            libs: [],
            ld_library_path: [],
            type: 'app',
            host: '',
            target: '',
            build_dir: 'build'
        }.merge(options)
    end
    
    def include_info
        @options[:include_path].find_all{|path| File.directory?(path) }.map{|path| "-I" + path }.join(" ")
    end

    def ld_info
        @options[:ld_library_path].find_all{|path| File.directory?(path) }.map{|path| "-L" + path }.join(" ")
    end

    def lib_info
        @options[:libs].map{|path| "-l" + path }.join(" ")
    end

    def gcc
        return @options[:host] + '-g++' if @options[:host] != ''
        return 'g++'
    end

    def source_files
        source_files = @options[:source_files].size == 0 ? Dir["*/**/*.cpp"] + Dir["*/**/*.c"] : @options[:source_files]
    end

    def chanage_source_to_obj file
        File.join(@options[:build_dir], 'obj', file.gsub(File.extname(file), '.o'))
    end

    def obj_files
        source_files.map{|file| chanage_source_to_obj(file) }
    end

    def create_makefile_and_make
        open @options[:makefile], 'w' do |f|
            f << ERB.new(IO.read(File.join(__dir__, "../resources/makefile.#{@options[:type]}.erb"))).result(binding)
        end
        system("make")
    end

    # set options
    def host platform
        @options[:host] = platform
    end

    def set key, value
        @options[key] = value
    end

    def add_sources files
        @options[:source_files].concat files
    end

    def add_include_path paths
        @options[:include_path].concat paths
    end

    def add_ld_library_path paths
        @options[:ld_library_path].concat paths
    end

    def add_libs libs
        @options[:libs].concat libs 
    end

end